#!/bin/sh
#
#	%Z% %M% %I% %E% %Q%
#
# Copyright (c) 2004 The Santa Cruz Operation, Inc.. All Rights Reserved.
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE SANTA CRUZ OPERATION INC.
# The copyright notice above does not evidence any actual or intended
# publication of such source code.
# 
# Written 07-Jan-2004 by Ronald Joe Record <rr@sco.com>
#
# tomcat start/stop script
# Usage: tomcat [ start | stop | enable | disable | restart | status ]
#
set -a
JSVC=
# Startup using jsvc not implemented on OpenServer
#JSVC=jsvc_
TOMCAT_HOME="/usr/lib/apache/tomcat"
TOMCAT_BIN="$TOMCAT_HOME/bin"
JAVA_HOME=/usr/java
CATALINA_HOME=/usr/lib/apache/tomcat
CATALINA_OUT=/usr/lib/apache/tomcat/logs/catalina.out
PID_DIR=/var/run
PIDFILE=$PID_DIR/jsvc.pid

[ -f /etc/default/tomcat ] && {
	. /etc/default/tomcat
}

# definitions
USAGE="Usage: %s [ start  | stop | enable | disable | restart | status ]"
LABEL="UX:`basename $0`"
CAT="tomcat.cat"

# paths to tomcat files, and a label
TOMCAT_LABEL="tomcat"

# more paths, and some wait timeouts
PATH="/bin:/usr/bin:/etc:/usr/gnu/bin"; export PATH
SCRIPTDIR="/etc/init.d"
SCRIPT="tomcat"
SCRIPTNUM="95"

OS=`uname -v`
case  $OS in
  7.1.*)
	TOMCAT_USER=nobody
	TOMCAT_START="$TOMCAT_BIN/startup.sh"
	TOMCAT_STOP="$TOMCAT_BIN/shutdown.sh"
	;;
   5.0.7|6.0.0)
	TOMCAT_USER=nouser
	TOMCAT_START="/bin/su -S -s /bin/sh -c $TOMCAT_BIN/startup.sh $TOMCAT_USER"
	TOMCAT_START="/usr/bin/sd tomcat"
	TOMCAT_STOP="/bin/su -S -s /bin/sh -c $TOMCAT_BIN/shutdown.sh $TOMCAT_USER"
	;;
esac

STARTUP_LINK=S${SCRIPTNUM}${SCRIPT}
# Work around for stopping tomcat by killing it ..
TOMCAT_PID_FILE=$PID_DIR/kill.$TOMCAT_LABEL.pid

# PS_STR is used to determine the tomcat PID
PS_STR="java.endorsed.dirs=.*/.*tomcat"


#######################################################################
### get tomcat process PID
### return 0 if process is not running
#######################################################################
get_tomcat_pid() {
        # Do NOT change cut -d" " -f2 to extract tomcat  PID
        # read OWNER PID JUNK does not work on a UW714 system.

        PID=`ps -aef | \
                grep "${PS_STR}" | \
                grep -v "grep ${PS_STR}" | \
                sed "s~  *~ ~g" | sed "s~^ ~~" | \
                cut -d" " -f2`

        echo "${PID}" >${TOMCAT_PID_FILE}
}

reset_owner() {
	if [ "${TOMCAT_USER}" != root ] ; then
           chown -R ${TOMCAT_USER} $TOMCAT_HOME/logs $TOMCAT_HOME/work \
                        $TOMCAT_HOME/temp $TOMCAT_HOME/conf \
                        $TOMCAT_HOME/webapps/ROOT 2>&1 > /dev/null
           chown ${TOMCAT_USER} $TOMCAT_HOME/server/webapps $TOMCAT_HOME/webapps
	fi
}

#############################################################################
#
# tomcat_start
#
# start the server daemon running
#
#############################################################################
tomcat_start() {

	# start the server
        # reset_owner

	get_tomcat_pid
	PID=`cat ${TOMCAT_PID_FILE}`
 	if [ -n "$PID" ] ; then
	   dspmsg $CAT 3 \
		"%s is already running. You may use the restart option ...\n" \
			 $TOMCAT_LABEL
	   return 0
	else
	   dspmsg $CAT 3 \
		"%s is not running. Starting %s ... " \
			 $TOMCAT_LABEL $TOMCAT_LABEL
	fi

	nohup $TOMCAT_START >> ${CATALINA_OUT} 2>&1
	RC=$?
	if [ $RC -eq 0 ] ; then
		dspmsg $CAT 4 "done.\n"
                # reset_owner
	else

		# execution of the server binary failed in some way
		echo ""
		dspmsg $CAT 6 \
			"%s could not be started\n" $TOMCAT_LABEL
                # reset_owner
	fi
	return $RC
}


#############################################################################
#
# tomcat_stop
#
# stop the server daemon if it is running
#
#############################################################################
tomcat_stop() {

	# signal the server to stop
	dspmsg $CAT 8 \
		"Stopping %s .. " $TOMCAT_LABEL
	# First check whether tomcat is running or not ..
	get_tomcat_pid
	PID=`cat ${TOMCAT_PID_FILE}`
 	if [ -z "$PID" ] ; then
	   dspmsg $CAT 8 \
		"%s is not running .." $TOMCAT_LABEL
	   RC=0
	else
	   nohup $TOMCAT_STOP >> ${CATALINA_OUT} 2>&1 
	   RC=$?
	   # Double check whether tomcat is running or not ..
	   get_tomcat_pid
	   PID1=`cat ${TOMCAT_PID_FILE}`
	   if [ $RC -eq 0 -a -z "${PID1}" ]  ; then
		# server is no longer running
		dspmsg $CAT 9 " done.\n"
	   else
		dspmsg $CAT 9 \
			"%s could not be stopped. Killing $PID1." $TOMCAT_LABEL
		kill -9 ${PID1}
		RC=$?
		get_tomcat_pid
		PID2=`cat ${TOMCAT_PID_FILE}`
 		if [ -z "$PID2" ] ; then
		   dspmsg $CAT 9 " done.\n"
		else
		   dspmsg $CAT 9 \
			"\n$PID2 could not be killed.\n" $TOMCAT_LABEL
		   RC=1
		fi
	   fi
	fi
	return $RC
}

#############################################################################
#
# jsvc_tomcat_stop
#
# Stop Tomcat using jsvc
#
#############################################################################
jsvc_tomcat_stop() {

  if [ -f $PIDFILE ]
  then
    dspmsg $CAT 8 \
    "Stopping %s ... " $TOMCAT_LABEL
    PID=`cat $PIDFILE`
    kill $PID
    dspmsg $CAT 9 "done.\n"
  else
    tomcat_stop
  fi
	
  return 0
}

#############################################################################
#
# jsvc_tomcat_start
#
# Start Tomcat using jsvc
#
#############################################################################
jsvc_tomcat_start() {
 
    CLASSPATH=$JAVA_HOME/lib/tools.jar:$CATALINA_HOME/bin/commons-daemon.jar:$CATALINA_HOME/bin/bootstrap.jar
    TMP_DIR=/tmp

    # reset_owner
    dspmsg $CAT 3 \
	"Starting %s ... " $TOMCAT_LABEL
    [ -d $PID_DIR ] || mkdir -p $PID_DIR
    $CATALINA_HOME/bin/jsvc \
    -user $TOMCAT_USER \
    -home $JAVA_HOME \
    -pidfile $PIDFILE \
    -Dcatalina.home=$CATALINA_HOME \
    -Djava.io.tmpdir=$TMP_DIR \
    -outfile $CATALINA_HOME/logs/catalina.out \
    -errfile '&1' \
    -cp $CLASSPATH \
    org.apache.catalina.startup.Bootstrap
    #
    # To get a verbose JVM
    #-verbose \
    # To get a debug of jsvc.
    #-debug \
    dspmsg $CAT 4 "done.\n"
    # reset_owner
    return 0
}

#############################################################################
#
# tomcat_restart
#
# restart the server daemon by sending it a SIGHUP
#
#############################################################################
tomcat_restart() {

    dspmsg $CAT 21 "Restarting .." $TOMCAT_LABEL
    tomcat_stop
    RC=$?
    if [ $? -eq 0 ] ; then
       tomcat_start
       RC=$?
    else
       dspmsg $CAT 21 " tomcat could not be stopped .." $TOMCAT_LABEL
    fi
    return $RC
}

#############################################################################
#
# tomcat_status
#
# status of the service
# 0 - service running
# 1 - service dead, but /var/run/  pid  file exists
# 2 - service dead, but /var/lock/ lock file exists
# 3 - service not running
#
#############################################################################
tomcat_status() {
	get_tomcat_pid
	PID=`cat ${TOMCAT_PID_FILE}`
 	if [ -n "$PID" ] ; then
	  dspmsg $CAT 6 "%s is running\n" $TOMCAT_LABEL
          RC=0
        else
	  dspmsg $CAT 6 "%s is not running\n" $TOMCAT_LABEL
          #exit 3
          RC=3
        fi
	return $RC
}

#############################################################################
#
# tomcat_enable
#
# add a link to this script into the /etc/rc2.d directory, then start
# the server
#
#############################################################################
tomcat_enable() {

	# Link in /etc/init.d/tomcat to /etc/rc[02].d/[SP] startup links.
	# Remove any pre-existing link.
	# Otherwise the hard link get out of sync, if /etc/init.d/tomcat
	# inode is changed. 

	if [ -f /etc/rc*.d/*$SCRIPTNUM$SCRIPT ]; then
		# one or more links already exists
		test -z "$ISL" && dspmsg $CAT 15 \
			"%s is already enabled.\n" $TOMCAT_LABEL
		ENABLE_STR="Re-freshing the links:"
	fi

	test -z "$ISL" && dspmsg $CAT 16 \
		"$ENABLE_STR %s enabled ... " $TOMCAT_LABEL

	# Create $STARTUP_LINK.  UW714 /etc/rc2 starts S* scripts.
	# OSR5 and 6 /etc/rc2 starts K* scripts.
	for DIR in	/etc/rc2.d /etc/rc0.d
	do
		rm -f $DIR/*$SCRIPTNUM$SCRIPT
		ln $SCRIPTDIR/$SCRIPT $DIR/$STARTUP_LINK
	done
	test -z "$ISL" && dspmsg $CAT 17 "done.\n"

	# start the server
	test -z "$ISL" && tomcat_restart
	return $?

}


#############################################################################
#
# tomcat_disable
#
# stop the server, then remove any links to this script in /etc/rc*.d
#
#############################################################################
tomcat_disable() {

	# stop the server
	tomcat_stop
        RC=$?

	if [ -f /etc/rc*.d/$STARTUP_LINK ] ; then
	   test -z "$ISL" && dspmsg $CAT 18 \
			"Disabling %s ... " $TOMCAT_LABEL

	   # remove the /etc/rc*.d startup links.
	   for DIR in	/etc/rc2.d /etc/rc0.d
	   do
		   rm -f $DIR/$STARTUP_LINK
	   done
	   test -z "$ISL" && dspmsg $CAT 19 "done.\n"
	else
	   # file does not exist
	   test -z "$ISL" && dspmsg $CAT 20 \
		"%s is already disabled.\n" $TOMCAT_LABEL
	fi
	return $RC
}
	
#############################################################################
# main()
#############################################################################

# check the usage of the script
if [ $# -gt 1 ]; then
	dspmsg $CAT 1 "$USAGE\n" `basename $0`
	exit 1
fi

# function to perform is in $1
function="$1"

# execute the routine for the requested function
case $function in
start)
	${JSVC}tomcat_start
	RC=$?
	;;
stop)
	${JSVC}tomcat_stop
	RC=$?
	;;
enable)
	tomcat_enable
	RC=$?
	;;
disable)
	tomcat_disable
	RC=$?
	;;
restart)
	tomcat_restart
	RC=$?
	;;
status)
	tomcat_status
	RC=$?
	;;
*)
	dspmsg $CAT 1 "$USAGE\n" `basename $0`
	exit 1		;;
esac

# done
exit $RC
